﻿#*******************************************************************************
#
# E M B E D D E D   W I Z A R D   P R O J E C T
#
#                                                Copyright (c) TARA Systems GmbH
#                                    written by Paul Banach and Manfred Schweyer
#
#
# PowerShell script for modification of the MPLABX project configuration file
# according to Embedded Wizard project settings (color format, screen
# orientation, generated source files)
#
#*******************************************************************************

# the window title
$host.ui.RawUI.WindowTitle = "Embedded Wizard Post Process - updating MPLAB project configuration"
$executingScriptDirectory  = Split-Path -Path $MyInvocation.MyCommand.Definition -Parent

# check for PowerShell version
$invalidPsVersion = 0
$psVersion        = $PSVersionTable.PSVersion
if ( $psVersion )
{
  Write-Host "PowerShell version: " $psVersion
  If($psVersion -lt '4.0') {
    $invalidPsVersion = 1;
  }
}
Else {
  Write-Host "No PowerShell Version found"
  $invalidPsVersion = 1;
}

if ( $invalidPsVersion ) {
  Write-Host This Embedded Wizard Post Process needs at least V4.0 of Windows Powershell
  Write-Host Powershell update can be found at:
  Write-Host https://docs.microsoft.com/de-de/powershell/scripting/setup/installing-windows-powershell
  Write-Host
  Read-Host Press Enter to continue | Out-Null
  [Environment]::Exit(10)
}

# get color format
$NewColorFormat = $env:EmWi_ColorFormat

# check, if requested color format is supported
$ppDir = Join-Path $executingScriptDirectory "../../../../PlatformPackage/$NewColorFormat"
if ( -Not ( Test-Path -Path $ppDir ))
{
  Write-Host "Selected color format" $NewColorFormat "not supported!"
  Read-Host Press Enter to continue | Out-Null
  [Environment]::Exit(8)
}


# get screen orientation
$ScreenOrientation     = $env:EmWi_SurfaceRotation
if ( $ScreenOrientation -eq 90 ) {
  $ScreenOrientation_ext = "-r90"
} elseif ( $ScreenOrientation -eq 180 ) {
  $ScreenOrientation_ext = "-r180"
} elseif ( $ScreenOrientation -eq 270 ) {
  $ScreenOrientation_ext = "-r270"
} else {
  $ScreenOrientation_ext = ""
}

# check for profesional version of Embedded Wizard Platform Package
$Path = Join-Path $executingScriptDirectory "../../../../PlatformPackage/$NewColorFormat/ewgfx.c"
$UseLibaries = 1
if (Test-Path $path)
{
  $UseLibaries = 0
}

# edit MPLABX project configuration file
$executingScriptDirectory = Split-Path -Path $MyInvocation.MyCommand.Definition -Parent
$Path = Join-Path $executingScriptDirectory "../firmware/EmbeddedWizard-PIC32MZ-DA-MEB-II.X/nbproject/configurations.xml"

if ( -Not ( Test-Path -Path $Path ))
{
  Write-Host MPLAB project configuration file not found at path:
  Write-Host $Path
  Write-Host
  Read-Host Press Enter to continue | Out-Null
  [Environment]::Exit(6)
}

# update color format
(Get-Content $Path).replace('RGBA8888', $NewColorFormat ) | Set-Content $Path
(Get-Content $Path).replace('RGBA4444', $NewColorFormat ) | Set-Content $Path
(Get-Content $Path).replace('RGB888', $NewColorFormat ) | Set-Content $Path
(Get-Content $Path).replace('RGB565', $NewColorFormat ) | Set-Content $Path
(Get-Content $Path).replace('LumA44', $NewColorFormat ) | Set-Content $Path
(Get-Content $Path).replace('Index8', $NewColorFormat ) | Set-Content $Path

# update screen orientation
(Get-Content $Path).replace('EW_SURFACE_ROTATION=0', 'EW_SURFACE_ROTATION=' + $ScreenOrientation ) | Set-Content $Path
(Get-Content $Path).replace('EW_SURFACE_ROTATION=90', 'EW_SURFACE_ROTATION=' + $ScreenOrientation ) | Set-Content $Path
(Get-Content $Path).replace('EW_SURFACE_ROTATION=180', 'EW_SURFACE_ROTATION=' + $ScreenOrientation ) | Set-Content $Path
(Get-Content $Path).replace('EW_SURFACE_ROTATION=270', 'EW_SURFACE_ROTATION=' + $ScreenOrientation ) | Set-Content $Path


$xml = [xml](get-content $Path)
if ( -Not $xml ) {
  Write-Host MPLAB project configuration file not found at path:
  Write-Host $Path
  Write-Host
  Read-Host Press Enter to continue | Out-Null
  [Environment]::Exit(6)
}


$xml.Load($Path)

# Select libary node
$ConfNodes = $xml.SelectNodes('//conf')


$libs = @(
  "../../../../../PlatformPackage/RTE/libewrte-mz-gcc.a",
  "../../../../../PlatformPackage/$NewColorFormat/libewgfx-mz-gcc.a",
  "../../../../../PlatformPackage/$NewColorFormat/libewgfx-mz-r90-gcc.a",
  "../../../../../PlatformPackage/$NewColorFormat/libewgfx-mz-r180-gcc.a",
  "../../../../../PlatformPackage/$NewColorFormat/libewgfx-mz-r270-gcc.a")


# Loop over all confs
foreach ( $conf in $ConfNodes ) {
  $target =  $conf.compileType.linkerTool.linkerLibItems
  if ( -Not $target ) {
    Write-Host linkerLibItems not found
    Read-Host Press Enter to continue | Out-Null
    exit 1
  }

  # remove all Embedded Wizard libraries
  for ($i=0; $i -lt $libs.length; $i++){
    $newlib = $libs[$i]
    do {
      $node = $target.SelectSingleNode("//linkerLibFileItem[.='$newlib']")
      if ( $node ) {
        $node.ParentNode.RemoveChild($node) | Out-Null
      }
    } while ( $node )
  }
}

# navigate to platform package node
$pp = $xml.configurationDescriptor
if ( $pp ) { $pp = $pp.logicalFolder | where {$_.displayName -eq "root"} }
if ( $pp ) { $pp = $pp.logicalFolder | where {$_.displayName -eq "Source Files"} }
if ( $pp ) { $pp = $pp.logicalFolder | where {$_.displayName -eq "app"} }
if ( $pp ) { $pp = $pp.logicalFolder | where {$_.displayName -eq "EmbeddedWizard"} }
if ( $pp ) { $pp = $pp.logicalFolder | where {$_.displayName -eq "PlatformPackage"} }

if ( -Not $pp ) {
  Write-Host PlatformPackage item not found
  Read-Host Press Enter to continue | Out-Null
  exit 1
}

# select rte node
$rte = $pp.logicalFolder | where {$_.displayName -eq "RTE"}
if ( -Not $rte ) {
  Write-Host RTE item not found
  Read-Host Press Enter to continue | Out-Null
  exit 1
}

# delete all rte source file items
($rte.ChildNodes | ? {$rte.itemPath -eq $_.InnerText}) | % {
    $rte.RemoveChild($_) | Out-Null
}

# select gfx node
$gfx = $pp.logicalFolder | where {$_.displayName -eq $NewColorFormat}
if ( -Not $gfx ) {
  Write-Host GFX item not found
  Read-Host Press Enter to continue | Out-Null
  exit 1
}

# delete all gfx source file items
($gfx.ChildNodes | ? {$gfx.itemPath -eq $_.InnerText}) | % {
    $gfx.RemoveChild($_) | Out-Null
}


if ( $UseLibaries )
{
  # Loop over all confs
  foreach ( $conf in $ConfNodes ) {
    # Select libary node
    $target =  $conf.compileType.linkerTool.linkerLibItems
    if ( -Not $target ) {
      Write-Host linkerLibItems not found
      Read-Host Press Enter to continue | Out-Null
      exit 1
    }

    # add the requested gfx library
    $xmlElt = $xml.CreateElement("linkerLibFileItem")
    $xmlText = $xml.CreateTextNode("../../../../../PlatformPackage/$NewColorFormat/libewgfx-mz$ScreenOrientation_ext-gcc.a")
    $xmlElt.AppendChild($xmlText) | Out-Null
    $target.AppendChild($xmlElt)

    # add the requested rte library
    $xmlElt = $xml.CreateElement("linkerLibFileItem")
    $xmlText = $xml.CreateTextNode("../../../../../PlatformPackage/RTE/libewrte-mz-gcc.a")
    $xmlElt.AppendChild($xmlText) | Out-Null
    $target.AppendChild($xmlElt)
  }

  # add ewextrte.c
  $xmlElt = $xml.CreateElement("itemPath")
  $xmlText = $xml.CreateTextNode("../../../../../PlatformPackage/RTE/ewextrte.c")
  $xmlElt.AppendChild($xmlText) | Out-Null
  $rte.AppendChild($xmlElt) | Out-Null

  # add ewextgfx.c
  $xmlElt = $xml.CreateElement("itemPath")
  $xmlText = $xml.CreateTextNode("../../../../../PlatformPackage/$NewColorFormat/ewextgfx.c")
  $xmlElt.AppendChild($xmlText) | Out-Null
  $gfx.AppendChild($xmlElt) | Out-Null
}
else
{
  # add platform package sources
  # add rte sources
  $RTESrcFiles = @("ewcolor.c", "ewdebug.c", "ewextrte.c", "ewobject.c", "ewpoint.c", "ewrect.c", "ewref.c", "ewresource.c", "ewscalars.c", "ewslot.c", "ewstring.c", "ewtimer.c", "ewheap.c", "ewinvocations.c" )
  foreach ($element in $RTESrcFiles) {
    $xmlElt = $xml.CreateElement("itemPath")
    $xmlText = $xml.CreateTextNode("../../../../../PlatformPackage/RTE/$element")
    $xmlElt.AppendChild($xmlText) | Out-Null
    $rte.AppendChild($xmlElt) | Out-Null
  }

  # add gfx sources
  $GFX_SrcFiles = @("ewextbmp_$NewColorFormat.c", "ewextfnt.c", "ewextgfx.c", "ewextpxl_$NewColorFormat.c", "ewgfx.c", "ewgfxattrtext.c", "ewgfxcore.c", "ewgfxdriver.c", "ewgfxtasks.c", "ewgfxbidi.c" )
  foreach ($element in $GFX_SrcFiles) {
    $xmlElt = $xml.CreateElement("itemPath")
    $xmlText = $xml.CreateTextNode("../../../../../PlatformPackage/$NewColorFormat/$element")
    $xmlElt.AppendChild($xmlText) | Out-Null
    $gfx.AppendChild($xmlElt) | Out-Null
  }
}


# navigate to generated source
$target = $xml.configurationDescriptor
if ( $target ) { $target = $target.logicalFolder | where {$_.displayName -eq "root"} }
if ( $target ) { $target = $target.logicalFolder | where {$_.displayName -eq "Source Files"} }
if ( $target ) { $target = $target.logicalFolder | where {$_.displayName -eq "app"} }
if ( $target ) { $target = $target.logicalFolder | where {$_.displayName -eq "EmbeddedWizard"} }
if ( $target ) { $target = $target.logicalFolder | where {$_.displayName -eq "GeneratedCode"} }

if ( -Not $target ) {
  Write-Host GeneratedCode item not found
  Read-Host Press Enter to continue | Out-Null
  exit 1
}

# delete all source files below GeneratedCode
($target.ChildNodes | ? {$target.itemPath -eq $_.InnerText}) | % {
    $target.RemoveChild($_) | Out-Null
}

# add generated source files
foreach ($element in ($env:EmWi_OutputSrcFiles -split '\s+')) {
  $xmlElt = $xml.CreateElement("itemPath")
  $xmlText = $xml.CreateTextNode("../../../../GeneratedCode/$element")
  $xmlElt.AppendChild($xmlText) | Out-Null
  $target.AppendChild($xmlElt) | Out-Null
}


#workaround: removing targetHeader element, since MPLAB has issues with it
foreach ( $conf in $ConfNodes ) {
  $target = $conf.toolsSet
  if ( $target ) {
    $node = $target.Item('targetHeader')
    if ($node) { $target.RemoveChild($node) | Out-Null }
  }
}

# Save and close MPLABX configuration file
$xml.Save($Path)

