/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template is responsible to initialize the system clock of the hardware,
*   to provide a timer tick for the Embedded Wizard UI application and to get
*   access to the realtime clock (RTC). Additionally, this file contains some
*   performance measurements to analyse the CPU usage of the UI application.
*
*******************************************************************************/

#include "ewconfig.h"
#include "ew_bsp_clock.h"
#include "system_config.h"
#include "system_definitions.h"


#define RtcTicksPerSecond    ( 1UL )
#define RtcTicksPerMinute    ( RtcTicksPerSecond * 60UL )
#define RtcTicksPerHour      ( RtcTicksPerMinute * 60UL )
#define RtcTicksPerDay       ( RtcTicksPerHour   * 24UL )

const unsigned long DaysToMonth[] =
{
  0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365
};

const unsigned long DaysToMonthInLeapYear[] =
{
  0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366
};



#if EW_CPU_LOAD_MEASURING == 1

  static unsigned long        WorkingTime  = 0;
  static unsigned long        SleepingTime = 0;
  static unsigned long        PrevCycCnt   = 0;

/* helper function to initialize measuring unit */
static void CycleCounterInit()
{
  /*

   ...

  */

  /* initialize counters */
  WorkingTime   = 0;
  SleepingTime  = 0;
  PrevCycCnt    = 0;
}

/* helper function to get counter value since last call */
static unsigned long GetCycleCounterDelta()
{
  unsigned long cycCnt;
  unsigned long result;

  cycCnt = DWT->CYCCNT;
  if ( cycCnt > PrevCycCnt )
    result = cycCnt - PrevCycCnt;
  else
    result = PrevCycCnt - cycCnt;
  PrevCycCnt = cycCnt;

  return result;
}

#endif


/*******************************************************************************
* FUNCTION:
*   EwBspClockInit
*
* DESCRIPTION:
*   Initialises the system clock and the real time clock.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspClockInit( void )
{
}


/*******************************************************************************
* FUNCTION:
*   EwBspClockGetTime
*
* DESCRIPTION:
*   Returns the current time in seconds since 01.01.1970.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   The current time in seconds since 01.01.1970.
*
*******************************************************************************/
unsigned long EwBspClockGetTime( void )
{
  SYS_RTCC_BCD_DATE  rtcDate;
  SYS_RTCC_BCD_TIME  rtcTime;

  if ( SYS_RTCC_DateGet( &rtcDate ) != SYS_RTCC_STATUS_OK )
    return 0;

  if ( SYS_RTCC_TimeGet( &rtcTime ) != SYS_RTCC_STATUS_OK )
    return 0;

  unsigned long   days;
  unsigned long   timeInSeconds;
  unsigned long   year, month, day, hour, minute, second;


  year   = (( rtcDate >> 28 ) & 0xF ) * 10 + (( rtcDate >> 24 ) & 0xF );
  month  = (( rtcDate >> 20 ) & 0xF ) * 10 + (( rtcDate >> 16 ) & 0xF );
  day    = (( rtcDate >> 12 ) & 0xF ) * 10 + (( rtcDate >>  8 ) & 0xF );
  hour   = (( rtcTime >> 28 ) & 0xF ) * 10 + (( rtcTime >> 24 ) & 0xF );
  minute = (( rtcTime >> 20 ) & 0xF ) * 10 + (( rtcTime >> 16 ) & 0xF );
  second = (( rtcTime >> 12 ) & 0xF ) * 10 + (( rtcTime >>  8 ) & 0xF );

  /* year is the number of years since 2000 */
  /* calculate number of days since 01.01.1970 until begin of current
     year including the additional days of leap years */

  days = (year + 30) * 365 + (year + 27) / 4;

  if (year & 0x3)
  {
    days += DaysToMonth[month - 1];
  }
  else
  {
    days += DaysToMonthInLeapYear[month - 1];
  }

  days += day;

  timeInSeconds =
    days   * RtcTicksPerDay +
    hour   * RtcTicksPerHour +
    minute * RtcTicksPerMinute +
    second;

  return timeInSeconds;
}


/*******************************************************************************
* FUNCTION:
*   EwBspClockSetTime
*
* DESCRIPTION:
*   Sets the given time in seconds since 01.01.1970 at real time clock (RTC).
*
* ARGUMENTS:
*   aTime - the time in seconds since 01.01.1970 to set in real time clock.
*
* RETURN VALUE:
*   None.
*
*******************************************************************************/
void EwBspClockSetTime( unsigned long aTime )
{
  #define LeapYearsFromBeginToNext400Year   7
  #define DaysPer4Years       ( 4 * 365 + 1 )

  SYS_RTCC_BCD_DATE  rtcDate;
  SYS_RTCC_BCD_TIME  rtcTime;

  unsigned int year;
  unsigned int year4;
  unsigned int month;
  unsigned int day;
  unsigned int hour;
  unsigned int minute;
  unsigned int second;

  hour    = ( unsigned int ) (( aTime / RtcTicksPerHour    ) % 24 );
  minute  = ( unsigned int ) (( aTime / RtcTicksPerMinute  ) % 60 );
  second  = ( unsigned int ) (( aTime / RtcTicksPerSecond  ) % 60 );

  /* calculation of the date */
  day       = ( unsigned int ) ( aTime / RtcTicksPerDay + 365);

  if ( day >= 48213 )
    day++;

  year4     = day / DaysPer4Years;
  day      -= year4 * DaysPer4Years;
  year      = day / 365;

  if ( year == 4 )
  {
    year = 3;

    if ( !( year4 % 100 == LeapYearsFromBeginToNext400Year )
       && ( year4 %  25 == LeapYearsFromBeginToNext400Year ))
      year = 4;
  }

  day  -= year * 365;
  year  = year4 * 4 + year + 1969;

  /* all months have less than 32 days, days >> 5 is a good estimate
     for the month. */

  month = day >> 5;

  /* correct month if necessary */
  if ( ( year % 4 != 0 ) ||
      (( year % 100 == 0 ) && ( year % 400 != 0 )) )
  {
    /* no leap year */
    if ( day >= DaysToMonth[ month + 1 ] )
      month++;

    day -= DaysToMonth[ month ];
  }
  else
  {
    /* leap year */
    if ( day >= DaysToMonthInLeapYear[ month + 1 ] )
      month++;

    day -= DaysToMonthInLeapYear[ month ];
  }

  memset( &rtcTime, 0, sizeof( rtcTime ));
  memset( &rtcDate, 0, sizeof( rtcDate ));

  day   += 1;
  month += 1;
  year  -= 2000;

  rtcDate   = ( year   / 10 ) << 28;
  rtcDate  |= ( year   % 10 ) << 24;
  rtcDate  |= ( month  / 10 ) << 20;
  rtcDate  |= ( month  % 10 ) << 16;
  rtcDate  |= ( day    / 10 ) << 12;
  rtcDate  |= ( day    % 10 ) <<  8;

  rtcTime   = ( hour   / 10 ) << 28;
  rtcTime  |= ( hour   % 10 ) << 24;
  rtcTime  |= ( minute / 10 ) << 20;
  rtcTime  |= ( minute % 10 ) << 16;
  rtcTime  |= ( second / 10 ) << 12;
  rtcTime  |= ( second % 10 ) <<  8;

  SYS_RTCC_DateSet( rtcDate );
  SYS_RTCC_TimeSet( rtcTime, 1 );
}


/*******************************************************************************
* FUNCTION:
*   EwBspClockGetCpuLoad
*
* DESCRIPTION:
*   Returns the current CPU load as percent value.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   The current CPU load.
*
*******************************************************************************/
 int EwBspClockGetCpuLoad( void )
{
  int cpuLoad = 0;

  #if EW_CPU_LOAD_MEASURING == 1

    /* get actual counter value */
    WorkingTime += GetCycleCounterDelta();

    /* calculate CPU load as percent value */
    cpuLoad = ( int ) ( WorkingTime / (( SleepingTime + WorkingTime ) / 100 ));

    /* clear accumulated counter values */
    WorkingTime  = 0;
    SleepingTime = 0;

  #endif

  return cpuLoad;
}


/*******************************************************************************
* FUNCTION:
*   EwBspClockCpuLoadSetActive
*
* DESCRIPTION:
*   Starts the CPU load counting. Call this function whenever CPU processing
*   is needed.
*
* ARGUMENTS:
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspClockCpuLoadSetActive( void )
{
  #if EW_CPU_LOAD_MEASURING == 1

    static char initialized = 0;

    if ( !initialized )
    {
      CycleCounterInit();
      initialized = 1;
    }

    SleepingTime += GetCycleCounterDelta();

  #endif
}


/*******************************************************************************
* FUNCTION:
*   EwBspClockCpuLoadSetIdle
*
* DESCRIPTION:
*   Stops the CPU load counting. Call this function whenever CPU processing is
*   currently not needed since the program execution is waiting for some system
*   event.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspClockCpuLoadSetIdle( void )
{
  #if EW_CPU_LOAD_MEASURING == 1

    WorkingTime += GetCycleCounterDelta();

  #endif
}


/* msy */
