/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This file contains an interface to the NANO2D hardware of the PIC32 target
*   in order to provide graphics acceleration for the Embedded Wizard generated
*   UI applications.
*   This interface is intended to be used only by the Graphics Engine of
*   Embedded Wizard.
*
*******************************************************************************/
#include "system_config.h"
#include "system_definitions.h"
#include "gfx/driver/processor/nano2d/libnano2D.h"

#include "ewrte.h"
#include "ewgfx.h"
#include "ewextgfx.h"
#include "ewgfxdefs.h"
#include "ew_bsp_graphics.h"
#include "ew_bsp_clock.h"


#if EW_USE_OPERATING_SYSTEM == 1
  #include "cmsis_os.h"
  osSemaphoreId AcceleratorSemaphoreId;
#endif


static volatile char       TransferInProgress  = 0;
static char                ConcurrentOperation = 1;
static unsigned long       BytePerPx[] =
  {
    4,  /* N2D_RGBA8888 */
    4,  /* N2D_BGRA8888 */
    2,  /* N2D_RGB565   */
    2,  /* N2D_BGR565   */
    2,  /* N2D_RGBA4444 */
    2,  /* N2D_BGRA4444 */
    1   /* N2D_A8       */
  };


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsInit
*
* DESCRIPTION:
*   Initialize the NANO2D graphics accelerator.
*
* ARGUMENTS:
*   aDstColorMode - Colorformat of the destination bitmap
*
* RETURN VALUE:
*   If successful, returns != 0.
*
*******************************************************************************/
int EwBspGraphicsInit( uint32_t aDstColorMode )
{
  /* libnano2d compatibility check */
    if ( N2D_RGBA8888 != 0 || N2D_BGRA8888 != 1 || N2D_RGB565 != 2
      || N2D_BGR565 != 3 || N2D_RGBA4444 != 4 || N2D_BGRA4444 != 5 || N2D_A8 != 6 )
    EW_PANIC( "libnano2d compatibility check failed" );

  #if EW_USE_OPERATING_SYSTEM == 1

    /* Create the accelerator semaphore */
    osSemaphoreDef(AcceleratorSemaphore);
    AcceleratorSemaphoreId = osSemaphoreCreate(osSemaphore(AcceleratorSemaphore), 1);

    /* initially take the accelerator token for the first NANO2D transfer */
    osSemaphoreWait( AcceleratorSemaphoreId, 1000 );

  #endif

  TransferInProgress  = 0;

  return 1;
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsDone
*
* DESCRIPTION:
*   Deinitialize the NANO2D graphics accelerator.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsDone( void )
{
  /* deinitialize the graphics hal */
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsWaitForCompletion
*
* DESCRIPTION:
*   The function EwBspGraphicsWaitForCompletion returns as soon as the NANO2D has
*   completed a pending graphics instruction. If the NANO2D is in idle mode,
*   the function returns immediately.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsWaitForCompletion()
{
  // currently all NANO2D transfer is done synchonous
  return;
#if 0
  /* return immediately if no NANO2D transfer is ongoing */
  if ( TransferInProgress == 0 )
    return;

  CPU_LOAD_SET_IDLE();


#ifdef EW_USE_NANO2D_INTERRUPT_MODE

  #if EW_USE_OPERATING_SYSTEM == 1

    osSemaphoreWait( AcceleratorSemaphoreId, 1000 );
    TransferInProgress = 0;

  #else

    /* wait until NANO2D transfer is done */
    while( TransferInProgress )
      ;

  #endif

#else

  /* wait until NANO2D transfer is done */
  //n2d_PollForTransfer();
  TransferInProgress = 0;

#endif

  CPU_LOAD_SET_ACTIVE();
#endif
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsConcurrentOperation
*
* DESCRIPTION:
*   The function EwBspGraphicsConcurrentOperation configures the operation mode
*   of NANO2D and CPU. If concurrent operation is enabled, the CPU will work in
*   parallel while the NANO2D is transferring data. If concurrent operation is
*   disabled, the CPU will wait everytime the NANO2D is active.
*   This feature is intended to limit the memory bandwidth, e.g. during display
*   update or other bandwidth consuming activities.
*
* ARGUMENTS:
*   aEnable - flag to switch on/off concurrent operation mode.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsConcurrentOperation( int aEnable )
{
  ConcurrentOperation = (char)aEnable;
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsFill
*
* DESCRIPTION:
*   The function EwBspGraphicsFill is used by the Graphics Engine, when a
*   rectangular area should be filled with a constant color by using the NANO2D
*   functionality.
*
* ARGUMENTS:
*   aDstAddr      - Destination address of the transfer.
*   aDstOffset    - Offset in pixel to next row within the destination.
*   aWidth,
*   aHeight       - Size of the area to fill.
*   aDstColorMode - Colorformat of the destination.
*   aSrcColor     - Color to fill the rectangular area.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsFill( uint32_t aDstAddr, uint32_t aDstWidth, uint32_t aDstHeight, uint32_t aDstColorMode,
  uint32_t aX, uint32_t aY, uint32_t aWidth, uint32_t aHeight, uint32_t aSrcColor )
{
  n2d_buffer_t    dst;
  n2d_rectangle_t rect;

  /* ensure that previous NANO2D instruction is finished */
  EwBspGraphicsWaitForCompletion();

  /* prepare GPU destination buffer structure */
  dst.format      = aDstColorMode;
  dst.gpu         = KVA_TO_PA( aDstAddr );
  dst.memory      = (void*)aDstAddr;
  dst.width       = aDstWidth;
  dst.height      = aDstHeight;
  dst.orientation = N2D_0;
  dst.stride      = dst.width * BytePerPx[ aDstColorMode ];

  rect.x      = aX;
  rect.y      = aY;
  rect.width  = aWidth;
  rect.height = aHeight;


  /* set the flag for NANO2D transfer */
  TransferInProgress = 1;

#ifdef EW_USE_NANO2D_INTERRUPT_MODE

  /* start the transfer */

#else

  /* process the transfer */
  if ( n2d_fill( &dst, &rect, aSrcColor, N2D_BLEND_NONE ) != N2D_SUCCESS )
    EW_ERROR( "n2d_fill failed" );

#endif

  /* check for immediate completion of drawing operation */
  if ( !ConcurrentOperation )
    EwBspGraphicsWaitForCompletion();
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsFillBlend
*
* DESCRIPTION:
*   The function EwBspGraphicsFillBlend is used by the Graphics Engine, when a
*   rectangular area should be blended with a constant color by using the NANO2D
*   functionality.
*
* ARGUMENTS:
*   aDstAddr      - Destination address of the transfer.
*   aDstOffset    - Offset in pixel to next row within the destination.
*   aWidth,
*   aHeight       - Size of the area to blend.
*   aDstColorMode - Colorformat of the destination.
*   aSrcColor     - Color to blend the rectangular area.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsFillBlend( uint32_t aDstAddr, uint32_t aDstWidth, uint32_t aDstHeight, uint32_t aDstColorMode,
  uint32_t aX, uint32_t aY, uint32_t aWidth, uint32_t aHeight, uint32_t aSrcColor )
{
  n2d_buffer_t    dst;
  n2d_rectangle_t rect;

  /* ensure that previous NANO2D instruction is finished */
  EwBspGraphicsWaitForCompletion();

  /* prepare GPU destination buffer structure */
  dst.format      = aDstColorMode;
  dst.gpu         = KVA_TO_PA( aDstAddr );
  dst.memory      = (void*)aDstAddr;
  dst.width       = aDstWidth;
  dst.height      = aDstHeight;
  dst.orientation = N2D_0;
  dst.stride      = dst.width * BytePerPx[ aDstColorMode ];

  rect.x      = aX;
  rect.y      = aY;
  rect.width  = aWidth;
  rect.height = aHeight;


  /* set the flag for NANO2D transfer */
  TransferInProgress = 1;

#ifdef EW_USE_NANO2D_INTERRUPT_MODE

  /* start the transfer */

#else

  if ( n2d_fill( &dst, &rect, aSrcColor, N2D_BLEND_SRC_OVER ) != N2D_SUCCESS )
    EW_ERROR( "n2d_fill failed" );

#endif

  /* check for immediate completion of drawing operation */
  if ( !ConcurrentOperation )
    EwBspGraphicsWaitForCompletion();
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsCopy
*
* DESCRIPTION:
*   The function EwBspGraphicsCopy is used by the Graphics Engine, when a source
*   bitmap should be copied into a destination bitmap by using the NANO2D
*   functionality.
*
* ARGUMENTS:
*   aDstAddr      - Destination address of the transfer.
*   aSrcAddr      - Source address of the transfer.
*   aDstOffset    - Offset in pixel to next row within the destination bitmap.
*   aSrcOffset    - Offset in pixel to next row within the source bitmap.
*   aWidth,
*   aHeight       - Size of the area to transfer the bitmap.
*   aDstColorMode - Colorformat of the destination bitmap
*   aSrcColorMode - Colorformat of the source bitmap.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsCopy(
  uint32_t aDstAddr, uint32_t aDstWidth, uint32_t aDstHeight, uint32_t aDstColorMode,
  uint32_t aSrcAddr, uint32_t aSrcWidth, uint32_t aSrcHeight, uint32_t aSrcColorMode,
  uint32_t aDstX, uint32_t aDstY,
  uint32_t aSrcX, uint32_t aSrcY, uint32_t aWidth, uint32_t aHeight )
{
  n2d_buffer_t    src;
  n2d_buffer_t    dst;
  n2d_rectangle_t rectSrc;
  n2d_rectangle_t rectDst;

  /* ensure that previous NANO2D instruction is finished */
  EwBspGraphicsWaitForCompletion();

  /* prepare GPU source buffer structure */
  src.format      = aSrcColorMode;
  src.gpu         = KVA_TO_PA( aSrcAddr );
  src.memory      = (void*)aSrcAddr;
  src.width       = aSrcWidth;
  src.height      = aSrcHeight;
  src.orientation = N2D_0;
  src.stride      = src.width * BytePerPx[ aSrcColorMode ];

  /* prepare GPU destination buffer structure */
  dst.format      = aDstColorMode;
  dst.gpu         = KVA_TO_PA( aDstAddr );
  dst.memory      = (void*)aDstAddr;
  dst.width       = aDstWidth;
  dst.height      = aDstHeight;
  dst.orientation = N2D_0;
  dst.stride      = dst.width * BytePerPx[ aDstColorMode ];

  rectSrc.x      = aSrcX;
  rectSrc.y      = aSrcY;
  rectSrc.width  = aWidth;
  rectSrc.height = aHeight;

  rectDst.x      = aDstX;
  rectDst.y      = aDstY;
  rectDst.width  = aWidth;
  rectDst.height = aHeight;

  /* set the flag for NANO2D transfer */
  TransferInProgress = 1;

#ifdef EW_USE_NANO2D_INTERRUPT_MODE

  /* start the transfer */

#else

  /* process the transfer */
  if ( n2d_blit( &dst, &rectDst, &src, &rectSrc, N2D_BLEND_NONE ) != N2D_SUCCESS )
    EW_ERROR( "n2d_blit failed" );

#endif

  /* check for immediate completion of drawing operation */
  if ( !ConcurrentOperation )
    EwBspGraphicsWaitForCompletion();
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsCopyBlend
*
* DESCRIPTION:
*   The function EwBspGraphicsCopyBlend is used by the Graphics Engine, when a
*   source bitmap should be blended over a destination bitmap by using the NANO2D
*   functionality.
*
* ARGUMENTS:
*   aDstAddr      - Destination address of the transfer.
*   aSrcAddr      - Source address of the transfer.
*   aDstOffset    - Offset in pixel to next row within the destination bitmap.
*   aSrcOffset    - Offset in pixel to next row within the source bitmap.
*   aWidth,
*   aHeight       - Size of the area to transfer the bitmap.
*   aDstColorMode - Colorformat of the destination bitmap
*   aSrcColorMode - Colorformat of the source bitmap.
*   aSrcColor     - Color/alpha value to be used for the transfer.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsBlend(
  uint32_t aDstAddr, uint32_t aDstWidth, uint32_t aDstHeight, uint32_t aDstColorMode,
  uint32_t aSrcAddr, uint32_t aSrcWidth, uint32_t aSrcHeight, uint32_t aSrcColorMode,
  uint32_t aDstX, uint32_t aDstY,
  uint32_t aSrcX, uint32_t aSrcY, uint32_t aWidth, uint32_t aHeight )
{
  n2d_buffer_t    src;
  n2d_buffer_t    dst;
  n2d_rectangle_t rectSrc;
  n2d_rectangle_t rectDst;

  /* ensure that previous NANO2D instruction is finished */
  EwBspGraphicsWaitForCompletion();

  /* prepare GPU source buffer structure */
  src.format      = aSrcColorMode;
  src.gpu         = KVA_TO_PA( aSrcAddr );
  src.memory      = (void*)aSrcAddr;
  src.width       = aSrcWidth;
  src.height      = aSrcHeight;
  src.orientation = N2D_0;
  src.stride      = src.width * BytePerPx[ aSrcColorMode ];

  /* prepare GPU destination buffer structure */
  dst.format      = aDstColorMode;
  dst.gpu         = KVA_TO_PA( aDstAddr );
  dst.memory      = (void*)aDstAddr;
  dst.width       = aDstWidth;
  dst.height      = aDstHeight;
  dst.orientation = N2D_0;
  dst.stride      = dst.width * BytePerPx[ aDstColorMode ];


  rectSrc.x      = aSrcX;
  rectSrc.y      = aSrcY;
  rectSrc.width  = aWidth;
  rectSrc.height = aHeight;

  rectDst.x      = aDstX;
  rectDst.y      = aDstY;
  rectDst.width  = aWidth;
  rectDst.height = aHeight;

  /* set the flag for NANO2D transfer */
  TransferInProgress = 1;

#ifdef EW_USE_NANO2D_INTERRUPT_MODE

  /* start the transfer */

#else

  /* process the transfer */
//  n2d_draw_state(  N2D_TRANSPARENCY_SOURCE, 0x80808080, 0xC, 0xC);
  if ( n2d_blit( &dst, &rectDst, &src, &rectSrc, N2D_BLEND_SRC_OVER ) != N2D_SUCCESS )
    EW_ERROR( "n2d_blit failed" );

#endif

  /* check for immediate completion of drawing operation */
  if ( !ConcurrentOperation )
    EwBspGraphicsWaitForCompletion();
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsCopyWithClut
*
* DESCRIPTION:
*   The function EwBspGraphicsCopyWithClut is used by the Graphics Engine, when
*   a Index8 source bitmap should be copied into a destination bitmap by using
*   the NANO2D functionality. The colors of the source bitmap are stored within
*   the given CLUT.
*
* ARGUMENTS:
*   aDstAddr      - Destination address of the transfer.
*   aSrcAddr      - Source address of the transfer.
*   aDstOffset    - Offset in pixel to next row within the destination bitmap.
*   aSrcOffset    - Offset in pixel to next row within the source bitmap.
*   aWidth,
*   aHeight       - Size of the area to transfer the bitmap.
*   aDstColorMode - Colorformat of the destination bitmap
*   aClutAddr     - Address of the first color entry of the CLUT.
*   aClutSize     - Number of color entries within the CLUT.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsCopyWithClut( uint32_t aDstAddr, uint32_t aSrcAddr, uint32_t aDstOffset,
  uint32_t aSrcOffset, uint32_t aWidth, uint32_t aHeight, uint32_t aDstColorMode,
  uint32_t aClutAddr, uint32_t aClutSize )
{
  /* ensure that previous NANO2D instruction is finished */
  EwBspGraphicsWaitForCompletion();

  /* set the flag for CLUT transfer */
  TransferInProgress = 1;

#ifdef EW_USE_NANO2D_INTERRUPT_MODE

  /* start the transfer */

#else

  /* process the transfer */

#endif

  /* wait for immediate completion of CLUT loading */
  EwBspGraphicsWaitForCompletion();

  /* set the flag for NANO2D transfer */
  TransferInProgress = 1;

#ifdef EW_USE_NANO2D_INTERRUPT_MODE

  /* start the transfer */

#else

  /* process the transfer */

#endif

  /* check for immediate completion of drawing operation */
  if ( !ConcurrentOperation )
    EwBspGraphicsWaitForCompletion();
}


#ifdef EW_USE_NANO2D_INTERRUPT_MODE

/*******************************************************************************
* FUNCTION:
*   NANO2D_IRQHandler
*
* DESCRIPTION:
*   NANO2D Interrupt Handler.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void NANO2D_IRQHandler( void )
{
  #if EW_USE_OPERATING_SYSTEM == 1

  osSemaphoreRelease( AcceleratorSemaphoreId );

  #else

  TransferInProgress = 0;

  #endif
}

#endif

/* msy, mli */
